/*
 * Accton AS5835 CPU CPLD Driver
 *
 * Copyright (c) 2019 Cumulus Networks, Inc.  All rights reserved.
 * Author: David Yen <dhyen@cumulusnetworks.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/i2c.h>
#include <linux/i2c-mux.h>
#include <linux/platform_data/at24.h>
#include <linux/i2c/sff-8436.h>
#include <linux/i2c/pca954x.h>
#include <linux/platform_device.h>
#include <linux/hwmon.h>
#include <linux/hwmon-sysfs.h>
#include <linux/cumulus-platform.h>

#include "platform_defs.h"
#include "platform_bitfield.h"
#include "accton_as5835.h"

#define DRIVER_NAME	   AS5835_CPUCPLD_NAME
#define DRIVER_VERSION	   "1.0"

/* bitfield accessor functions */

#define cpld_read_reg  bf_i2c_read_reg
#define cpld_write_reg bf_i2c_write_reg

/* cpld register bitfields with enum-like values */

static const char * const cpu_id_values[] = {
	"C3958",
	"C3558",
};

/* CPLD registers */

cpld_bt_ro(cpu_id, ACCTON_AS5835_CPUCPLD_BOARD_INFO_REG,
	   ACCTON_AS5835_CPUCPLD_CPU_ID, cpu_id_values, 0)
cpld_bf_ro(pcb_version, ACCTON_AS5835_CPUCPLD_BOARD_INFO_REG,
	   ACCTON_AS5835_CPUCPLD_PCB_VERSION, NULL, 0)
cpld_bf_ro(cpucpld_version, ACCTON_AS5835_CPUCPLD_VERSION_REG,
	   ACCTON_AS5835_CPUCPLD_VERSION, NULL, 0)
cpld_bt_rw(system_reset, ACCTON_AS5835_CPUCPLD_SYSTEM_STATE_CONTROL_REG,
	   ACCTON_AS5835_CPUCPLD_SYSTEM_RESET, NULL, 0)

/* sysfs registration */

static struct attribute *cpld_attrs[] = {
	&cpld_cpu_id.attr,
	&cpld_pcb_version.attr,
	&cpld_cpucpld_version.attr,
	&cpld_system_reset.attr,

	NULL,
};

static struct attribute_group cpld_attr_group = {
	.attrs = cpld_attrs,
};

static int cpld_probe(struct i2c_client *client,
		      const struct i2c_device_id *id)
{
	s32 temp;
	int ret = 0;
	struct kobject *kobj = &client->dev.kobj;

	temp = i2c_smbus_read_byte_data(client,
					ACCTON_AS5835_CPUCPLD_VERSION_REG);
	if (temp < 0) {
		pr_err(DRIVER_NAME ": read CPU CPLD version register error: %d\n",
		       ret);
		return temp;
	}

	ret = sysfs_create_group(kobj, &cpld_attr_group);
	if (ret)
		pr_err(DRIVER_NAME ": failed to create sysfs group for cpld device\n");

	pr_info(DRIVER_NAME ": device probed, CPU CPLD rev: %d\n",
		(int)GET_FIELD(temp, ACCTON_AS5835_CPUCPLD_VERSION));

	return ret;
}

static int cpld_remove(struct i2c_client *client)
{
	struct kobject *kobj = &client->dev.kobj;

	sysfs_remove_group(kobj, &cpld_attr_group);

	pr_info(DRIVER_NAME ": cpld device removed\n");
	return 0;
}

static const struct i2c_device_id cpld_i2c_ids[] = {
	{
		.name = DRIVER_NAME,
	},
	{ /* end of list */ },
};

MODULE_DEVICE_TABLE(i2c, cpld_i2c_ids);

static struct i2c_driver cpld_driver = {
	.driver = {
		.name = DRIVER_NAME,
		.owner = THIS_MODULE,
	},
	.probe = cpld_probe,
	.remove = cpld_remove,
	.id_table = cpld_i2c_ids,
};

/* module init and exit */

static int __init cpld_init(void)
{
	int ret;

	ret = i2c_add_driver(&cpld_driver);
	if (ret) {
		pr_err(DRIVER_NAME ": driver failed to load\n");
		return ret;
	}

	pr_info(DRIVER_NAME ": version " DRIVER_VERSION " loaded\n");
	return 0;
}

static void __exit cpld_exit(void)
{
	i2c_del_driver(&cpld_driver);
	pr_info(DRIVER_NAME ": version " DRIVER_VERSION " unloaded\n");
}

module_init(cpld_init);
module_exit(cpld_exit);

MODULE_AUTHOR("David Yen (dhyen@cumulusnetworks.com)");
MODULE_DESCRIPTION("Accton AS5835 CPU CPLD Driver");
MODULE_VERSION(DRIVER_VERSION);
MODULE_LICENSE("GPL");
